package MT::Plugin::RandomFeed;
use strict;
use MT;
use MT::Plugin;
use base qw( MT::Plugin );
use MT::Util qw( encode_html );

use Encode;

my $plugin = new MT::Plugin::RandomFeed ( {
    description => 'Show random message from feed to User dashboard.',
    name => 'RandomFeed',
    id   => 'RandomFeed',
    key  => 'randomfeed',
    author_name => 'Junnama Noda',
    author_link => 'http://junnama.alfasado.net/online/',
    version => '0.1',
    settings => new MT::PluginSettings( [
        [ 'feed_url_for_user_dashboard_1',
            { Default => 'http://twitter.com/statuses/user_timeline/6782492.rss' } ],
        [ 'feed_url_for_user_dashboard_2',
            { Default => 'http://twitter.com/statuses/user_timeline/63097969.rss' } ],
        [ 'feed_url_for_user_dashboard_3',
            { Default => 'http://twitter.com/statuses/user_timeline/10550172.rss' } ],
        [ 'feed_url_for_user_dashboard_4',
            { Default => 'http://junnama.alfasado.net/online/atom.xml' } ],
        [ 'feed_url_for_user_dashboard_5',
            { Default => 'http://twitter.com/statuses/user_timeline/64418903.rss' } ],
    ] ),
    system_config_template => 'config.tmpl',
} );

MT->add_plugin( $plugin );

sub init_registry {
    my $plugin = shift;
    $plugin->registry( {
        callbacks => {
            'MT::App::CMS::template_source.dashboard' => \&_randomfeed,
        },
   } );
}

sub _randomfeed {
    my ( $cb, $app, $tmpl ) = @_;
    if ( ( $app->view eq 'user' ) || ( $app->mode eq 'default' ) ) {
        my @url;
        for ( 1 .. 5 ) {
            my $uri = $plugin->get_config_value( 'feed_url_for_user_dashboard_' . $_ );
            push ( @url, $uri ) if $uri;
        }
        my $uri = $url[ int( rand( scalar @url ) ) ];
        my $get_feed = get_feed( $uri, 1 );
        my $lite = $get_feed->{ LITE };
        my $entries = $get_feed->{ ENTRIES };
        my $title = $get_feed->{ TITLE };
        my $link = $get_feed->{ LINK };
        my $count = $get_feed->{ COUNT };
        my $number = int( rand( $count ) );
        my $entry_title = $lite->find_title( $entries->[$number] );
        my $entry_link = $lite->find_link( $entries->[$number] );
        my $view_link =<<__EOT__;
        <a href="$entry_link" title="<__trans phrase="View">"
        target="_blank">
        <img src="<mt:var name="static_uri">images/status_icons/view.gif" alt="<__trans phrase="View">" />
        </a>
__EOT__
        $entry_title = encode_html( $entry_title );
        $entry_title =~ s/(s?https?:\/\/[-_.!~*'()a-zA-Z0-9;\/?:\@&=+\$,%#]+)/<a href="$1" target="_blank">$1<\/a>/g;
        my $title  =<<__EOT__;
        <mt:setvarblock name="page_title"><span style=\"font-size:14px\">
        <a href="$link" target="_blank">$title</a></span><br />$entry_title$view_link</mt:setvarblock>
__EOT__
        $$tmpl =~ s/(<mt:setvarblock\sname="html_head"\sappend="1">)/$1$title/;
    }
}

sub get_feed {
    my ( $uri, $utf8 ) = @_;
    return undef unless $uri;
    require MT::Feeds::Lite;
    my $lite  = MT::Feeds::Lite->fetch( $uri ) or return ''; 
    my $title = $lite->find_title( $lite->feed );
    my $link  = $lite->find_link ( $lite->feed );
    my $entries = $lite->entries;
    my $count = scalar @$entries;
    if ( $utf8 ) {
        $title = to_utf8( $title );
        $link  = to_utf8( $link );
        unless ( MT->version_number < 5 ) {
            $title = utf8_on( $title );
            $link  = utf8_on( $link );
        }
    }
    my %feed = (
        LITE    => $lite,
        TITLE   => $title,
        LINK    => $link,
        ENTRIES => \%$entries,
        COUNT   => $count,
    );
    return \%feed;
}

sub utf8_on {
    my $text = shift;
    if (! Encode::is_utf8( $text ) ) {
        Encode::_utf8_on( $text );
    }
    return $text;
}

sub to_utf8 {
    my $text = shift;
    return MT::I18N::encode_text( $text, undef, 'utf-8' );
}

1;